@@ -27,10 +27,24 @@
 
 ;;; Code:
 
+;; All glory to the byte compiler.
 (eval-when-compile (require 'cl))
-(provide 'reftex-auc)
+(eval-and-compile
+  (unless (fboundp 'declare-function) (defmacro declare-function (&rest r))))
+(declare-function TeX-argument-prompt "ext:tex"
+		  (optional prompt default &optional complete))
+(declare-function TeX-argument-insert "ext:tex"
+		  (name optional &optional prefix))
+(declare-function LaTeX-add-labels "ext:tex" (&rest entries))
+(declare-function LaTeX-add-index-entries "ext:tex" (&rest entries))
+(declare-function LaTeX-bibitem-list "ext:tex" ())
+(declare-function LaTeX-index-entry-list "ext:tex" ())
+(declare-function LaTeX-label-list "ext:tex" ())
+(declare-function multi-prompt "ext:multi-prompt"
+		  (separator unique prompt table &optional
+			     mp-predicate require-match initial history))
+
 (require 'reftex-base)
-;;;
 
 (defun reftex-plug-flag (which)
   ;; Tell if a certain flag is set in reftex-plug-into-AUCTeX
@@ -215,5 +229,7 @@ of ENTRY-LIST is a list of cons cells (\"MACRONAME\" . LEVEL).  See
 (defun reftex-notice-new-section ()
   (reftex-notice-new 1 'force))
 
+(provide 'reftex-auc)
+
 ;;; arch-tag: 4a798e68-3405-421c-a09b-0269aac64ab4
 ;;; reftex-auc.el ends here
