@@ -27,11 +27,13 @@
 
 ;;; Code:
 
+;; All glory to the byte compiler.
 (eval-when-compile (require 'cl))
-(provide 'reftex-dcr)
-(provide 'reftex-vcr)
+(eval-and-compile
+  (unless (fboundp 'declare-function) (defmacro declare-function (&rest r))))
+(declare-function bibtex-beginning-of-entry "bibtex" ())
+
 (require 'reftex-base)
-;;;
 
 (defun reftex-view-crossref (&optional arg auto-how fail-quietly)
   "View cross reference of macro at point.  Point must be on the KEY
@@ -230,6 +232,7 @@ If it is a \\cite, show the BibTeX database entry.
 If there is no such macro at point, search forward to find one.
 With argument, actually select the window showing the cross reference."
   (interactive "e")
+  ;; Make sure the referencing macro stays visible in the original window.
   (mouse-set-point ev)
   (reftex-view-crossref current-prefix-arg))
 
@@ -351,7 +354,8 @@ will display info in the echo area."
     (message "Automatic display of crossref information was turned on")))
 
 (defun reftex-start-itimer-once ()
-   (and reftex-mode
+   (and (featurep 'xemacs)
+	reftex-mode
         (not (itimer-live-p reftex-auto-view-crossref-timer))
         (setq reftex-auto-view-crossref-timer
               (start-itimer "RefTeX Idle Timer"
@@ -482,5 +486,7 @@ Calling this function several times find successive citation locations."
       (move-marker reftex-global-search-marker nil)
       (error "All files processed"))))
 
+(provide 'reftex-dcr)
+
 ;;; arch-tag: d2f52b56-744e-44ad-830d-1fc193b90eda
 ;;; reftex-dcr.el ends here
