@@ -1,7 +1,7 @@
 ;;; reftex-sel.el --- the selection modes for RefTeX
 
 ;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007 Free Software Foundation, Inc.
+;;   2006, 2007, 2008 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <dominik@science.uva.nl>
 ;; Maintainer: auctex-devel@gnu.org
@@ -420,7 +420,7 @@ During a selection process, these are the local bindings.
           (remove-hook 'post-command-hook 
                        'reftex-select-post-command-hook t))
         ;; Kill the mark overlays
-        (mapcar (lambda (c) (reftex-delete-overlay (nth 1 c)))
+        (mapc (lambda (c) (reftex-delete-overlay (nth 1 c)))
                 reftex-select-marked)))))
 
     (set (make-local-variable 'reftex-last-line)
@@ -533,21 +533,29 @@ Useful for large TOC's."
   (interactive)
   (setq reftex-last-follow-point -1)
   (setq cb-flag (not cb-flag)))
-(defun reftex-select-toggle-varioref ()
-  "Toggle the macro used for referencing the label between \\ref and \\vref."
-  (interactive)
-  (if (string= refstyle "\\ref")
-      (setq refstyle "\\vref")
-    (setq refstyle "\\ref"))
+
+(defun reftex-select-cycle-ref-style-internal (&optional reverse)
+  "Cycle through macros used for referencing.
+Cycle in reverse order if optional argument REVERSE is non-nil."
+  (let (list)
+    (dolist (style (reftex-ref-style-list))
+      (mapc (lambda (x) (add-to-list 'list (car x) t))
+	    (nth 2 (assoc style reftex-ref-style-alist))))
+    (when reverse
+      (setq list (reverse list)))
+    (setq refstyle (or (cadr (member refstyle list)) (car list))))
   (force-mode-line-update))
-(defun reftex-select-toggle-fancyref ()
-  "Toggle the macro used for referencing the label between \\ref and \\vref."
+
+(defun reftex-select-cycle-ref-style-forward ()
+  "Cycle forward through macros used for referencing."
   (interactive)
-  (setq refstyle
-        (cond ((string= refstyle "\\ref") "\\fref")
-              ((string= refstyle "\\fref") "\\Fref")
-              (t "\\ref")))
-  (force-mode-line-update))
+  (reftex-select-cycle-ref-style-internal))
+
+(defun reftex-select-cycle-ref-style-backward ()
+  "Cycle backward through macros used for referencing."
+  (interactive)
+  (reftex-select-cycle-ref-style-internal t))
+
 (defun reftex-select-show-insertion-point ()
   "Show the point from where selection was started in another window."
   (interactive)
@@ -642,13 +650,13 @@ Useful for large TOC's."
     (and ovl (reftex-delete-overlay ovl))
     (setq reftex-select-marked (delq cell reftex-select-marked))
     (setq cnt (1+ (length reftex-select-marked)))
-    (mapcar (lambda (c)
-              (setq sep (nth 2 c))
-              (reftex-overlay-put (nth 1 c) 'before-string
-                                  (if sep
-                                      (format "*%c%d* " sep (decf cnt))
-                                    (format "*%d*  " (decf cnt)))))
-            reftex-select-marked)
+    (mapc (lambda (c)
+	    (setq sep (nth 2 c))
+	    (reftex-overlay-put (nth 1 c) 'before-string
+				(if sep
+				    (format "*%c%d* " sep (decf cnt))
+				  (format "*%d*  " (decf cnt)))))
+	  reftex-select-marked)
     (message "Entry no longer marked")))
 
 (defun reftex-select-help ()
@@ -709,8 +717,8 @@ Useful for large TOC's."
 (loop for x in
       '(("b"        . reftex-select-jump-to-previous)
         ("z"        . reftex-select-jump)
-        ("v"        . reftex-select-toggle-varioref)
-        ("V"        . reftex-select-toggle-fancyref)
+        ("v"        . reftex-select-cycle-ref-style-forward)
+        ("V"        . reftex-select-cycle-ref-style-backward)
         ("m"        . reftex-select-mark)
         ("u"        . reftex-select-unmark)
         (","        . reftex-select-mark-comma)
