@@ -31,17 +31,17 @@
 
 ;;; Code:
 
-(defconst reftex-version "4.33"
+(defconst reftex-version "4.34"
   "Version string for RefTeX.")
 
 ;;; Autoloads
 
 
 ;;;### (autoloads (reftex-add-label-environments) "reftex-auc" "reftex-auc.el"
-;;;;;;  (18125 64963))
+;;;;;;  (19069 43000))
 ;;; Generated autoloads from reftex-auc.el
 
-(autoload (quote reftex-add-label-environments) "reftex-auc" "\
+(autoload 'reftex-add-label-environments "reftex-auc" "\
 Add label environment descriptions to `reftex-label-alist-style'.
 The format of ENTRY-LIST is exactly like `reftex-label-alist'.  See there
 for details.
@@ -54,21 +54,21 @@ the label information is recompiled on next use.
 
 \(fn ENTRY-LIST)" nil nil)
 
-(defalias (quote reftex-add-to-label-alist) (quote reftex-add-label-environments))
+(defalias 'reftex-add-to-label-alist 'reftex-add-label-environments)
 
 ;;;***
 
 ;;;### (autoloads (reftex-reset-scanning-information reftex-set-cite-format
 ;;;;;;  reftex-mode turn-on-reftex) "reftex-base" "reftex-base.el"
-;;;;;;  (18125 64963))
+;;;;;;  (19069 47710))
 ;;; Generated autoloads from reftex-base.el
 
-(autoload (quote turn-on-reftex) "reftex-base" "\
+(autoload 'turn-on-reftex "reftex-base" "\
 Turn on RefTeX mode.
 
 \(fn)" nil nil)
 
-(autoload (quote reftex-mode) "reftex-base" "\
+(autoload 'reftex-mode "reftex-base" "\
 Minor mode with distinct support for \\label, \\ref and \\cite in LaTeX.
 
 \\<reftex-mode-map>A Table of Contents of the entire (multifile) document with browsing
@@ -101,7 +101,7 @@ on the menu bar.
 
 \(fn &optional ARG)" t nil)
 
-(autoload (quote reftex-set-cite-format) "reftex-base" "\
+(autoload 'reftex-set-cite-format "reftex-base" "\
 Set the document-local value of `reftex-cite-format'.
 When such a value exists, it overwrites the setting given with
 `reftex-cite-format'.  See the documentation of `reftex-cite-format'
@@ -109,7 +109,7 @@ for possible values.  This function should be used from AUCTeX style files.
 
 \(fn VALUE)" nil nil)
 
-(autoload (quote reftex-reset-scanning-information) "reftex-base" "\
+(autoload 'reftex-reset-scanning-information "reftex-base" "\
 Reset the symbols containing information from buffer scanning.
 This enforces rescanning the buffer on next use.
 
@@ -118,10 +118,10 @@ This enforces rescanning the buffer on next use.
 ;;;***
 
 ;;;### (autoloads (reftex-citation) "reftex-cite" "reftex-cite.el"
-;;;;;;  (18125 64963))
+;;;;;;  (19024 29391))
 ;;; Generated autoloads from reftex-cite.el
 
-(autoload (quote reftex-citation) "reftex-cite" "\
+(autoload 'reftex-citation "reftex-cite" "\
 Make a citation using BibTeX database files.
 After prompting for a regular expression, scans the buffers with
 bibtex entries (taken from the \\bibliography command) and offers the
@@ -148,10 +148,10 @@ While entering the regexp, completion on knows citation keys is possible.
 ;;;***
 
 ;;;### (autoloads (reftex-isearch-minor-mode) "reftex-global" "reftex-global.el"
-;;;;;;  (18125 64963))
+;;;;;;  (18831 3226))
 ;;; Generated autoloads from reftex-global.el
 
-(autoload (quote reftex-isearch-minor-mode) "reftex-global" "\
+(autoload 'reftex-isearch-minor-mode "reftex-global" "\
 When on, isearch searches the whole document, not only the current file.
 This minor mode allows isearch to search through all the files of
 the current TeX document.
@@ -165,10 +165,10 @@ With no argument, this command toggles
 ;;;***
 
 ;;;### (autoloads (reftex-index-phrases-mode) "reftex-index" "reftex-index.el"
-;;;;;;  (18125 64962))
+;;;;;;  (19069 42983))
 ;;; Generated autoloads from reftex-index.el
 
-(autoload (quote reftex-index-phrases-mode) "reftex-index" "\
+(autoload 'reftex-index-phrases-mode "reftex-index" "\
 Major mode for managing the Index phrases of a LaTeX document.
 This buffer was created with RefTeX.
 
@@ -198,10 +198,10 @@ Here are all local bindings.
 ;;;***
 
 ;;;### (autoloads (reftex-all-document-files) "reftex-parse" "reftex-parse.el"
-;;;;;;  (18125 64962))
+;;;;;;  (18460 11748))
 ;;; Generated autoloads from reftex-parse.el
 
-(autoload (quote reftex-all-document-files) "reftex-parse" "\
+(autoload 'reftex-all-document-files "reftex-parse" "\
 Return a list of all files belonging to the current document.
 When RELATIVE is non-nil, give file names relative to directory
 of master file.
@@ -210,7 +210,7 @@ of master file.
 
 ;;;***
 
-;;;### (autoloads nil "reftex-vars" "reftex-vars.el" (18125 64962))
+;;;### (autoloads nil "reftex-vars" "reftex-vars.el" (19069 43010))
 ;;; Generated autoloads from reftex-vars.el
 (put 'reftex-vref-is-default 'safe-local-variable (lambda (x) (or (stringp x) (symbolp x))))
 (put 'reftex-fref-is-default 'safe-local-variable (lambda (x) (or (stringp x) (symbolp x))))
